############################################################
#
# Chatologica http library
# Hauptprozeduren:
#  HTTPGet - holt eine URL unter Verwendung der GET-Methode
#  HTTPPost - holt eine URL unter Verwendung der POST-Methode
#
# Setzen Sie die Variable $http_os auf 'NT', wenn Sie PERL auf einem
# Windows NT-Server laufen haben.
# Setzen Sie sie auf 'UNIX' fuer normale UNIX-Operationen.
#
# Wenn Ihre PERL-Version nicht ueber Socket.pm verfuegt, koennen Sie
# von Hand $AF_INET and $SOCK_STREAM auf  2 bzw. 1 definieren.
# Auf einigen Systemen mag gelten: SOCK_STREAM = 2.
#
############################################################

# Verwendung der Socket-Bibliothek fr TCP/IP Kommunikationen
use Socket;

# Betriebssystem ist Solaris, also nicht NT
$http_os = "UNIX";

############################################################
#
# subroutine: HTTPGet
#   Verwendung:
#	$buf = &HTTPGet("/index.html", "www.eff.org", 80,
#	"var1=value1&var2=value2&etc=etc");
#
#   Parameter:
#	$url = URL, zu der verbunden werden soll (relativ zum Host)
#	$hostname = Hostname der zu kontaktierenden WebSite 
#	$port = Port fr Verbindung (normalerweise 80)
#	$in = Formularwerte, die an die Site gesendet werden, um Output zu erhalten
#
#   Ausgabe:
#	HTML Code - Ausgabe des Webservers.
#
############################################################

sub HTTPGet
{
	local($url, $hostname, $port, $in) = @_;
	local($form_vars, $x, $socket);
	local ($buf);
	$socket = &OpenSocket($hostname, $port);

	$form_vars = &FormatFormVars($in);
	# Das Fragezeichen (?) muss vor uebergebenen Formularvariablen stehen.
	$url .= "?" . $form_vars;

	# Das folgende sendet Informationen an den HTTP-Server.
	print  $socket <<__END_OF_SOCKET__;
GET $url HTTP/1.1
Host: $hostname
Accept: text/html
Accept: text/plain
User-Agent: Mozilla/4.0


__END_OF_SOCKET__
   
	# RetrieveHTTP holt den HTML-Code ein.
	$buf = &RetrieveHTTP($socket);

	# Rueckgabewert
	$buf;

} # Ende der Subroutine HTTPGet


############################################################
#
# subroutine: HTTPPost
#   Verwendung:
#	$buf = &HTTPPost("/index.html", "www.eff.org", 80,
#	"var1=value1&var2=value2&etc=etc");
#
#   Parameter:
#	$url = URL, zu der verbunden werden soll (relativ zum Host)
#	$hostname = Hostname der zu kontaktierenden WebSite 
#	$port = Port fr Verbindung (normalerweise 80)
#	$in = Formularwerte, die an die Site gesendet werden, um Output zu erhalten
#
#   Ausgabe:
#	HTML Code - Ausgabe des Webservers.
#
############################################################

sub HTTPPost
{
	local($url, $hostname, $port, $in) = @_;
	local($form_vars, $x, $socket);
	local ($buf, $form_var_length);
	$socket = &OpenSocket($hostname, $port);

	# Das folgende sendet Informationen an den HTTP-Server.

	$form_vars = &FormatFormVars($in);

	# Die Laenge der zum Server uebermittelten Formularvariablen 
	# wird benoetigt ( = content-length)
	$form_var_length = length($form_vars);

	# Socketinformationen
	print $socket <<__END_OF_SOCKET__;
POST $url HTTP/1.1
Host: $hostname 
Accept: text/html
Accept: text/plain
User-Agent: Mozilla/4.0
Content-type: application/x-www-form-urlencoded
Content-length: $form_var_length

$form_vars
__END_OF_SOCKET__

	# URL-Inhalt einholen
	$buf = &RetrieveHTTP($socket);

	# Rueckgabewert
	$buf;

} # Ende der Subroutine HTTPPost


############################################################
#
# subroutine: FormatFormVars
#   Verwendung:
#	$formvars = &FormatFormVars($in);
#
#   Parameter:
#	$in = Zu verarbeitende Formularvariable
#
#   Ausgabe:
#	Verarbeitete Formularvariable. Unzulaessige Zeichen werden durch 
#	ihre HEX-Aequivalente ersetzt. (Im Augenblick nur fuer Leerzeichen
#	implementiert, kann aber erweitert werden.)
#
############################################################

sub FormatFormVars
{
	local ($in) = @_; 

	$in =~ s/ /%20/g;

	$in;
} # Ende der Subroutine FormatFormVars


############################################################
#
# subroutine: RetrieveHTTP
#   Verwendung:
#	$buf = &RetrieveHTTP($socket_handle);
#
#   Parameters:
#	$socket = Handle zum Socket, mit dem wir kommunizieren
#
#   Ausgabe:
#	Der Puffer enthaelt die Ausgabe des HTTP-Servers
#
############################################################

sub RetrieveHTTP
{
	local ($socket) = @_;
	local ($buf,$x, $split_length);

	$buf = read_sock($socket, 6);
	# Wenn der Puffer den Statuscode 200 enthaelt, ist es sicher,
	# den Rest des Dokuments zu lesen.

	if ($buf =~ /200/)
	{
		while(<$socket>)
		{
			$buf .= $_;
		}
	}

	# Der HTTP-Header wird entfernt, indem nach zwei Zeilenumbruechen
	# bzw. Zeilenumbruechen mit Wagenruecklaefen Ausschau gehalten
	# wird.
	# Auf verschiedenen Servern findet man manchmal unterschiedliche
	# Trennzeichen.

	$x = index($buf, "\r\n\r\n");
	$split_length = 4;

	if ($x == -1)
	{
		$x = index($buf, "\n\n");
		$split_length = 2;
	}
	
	# Nun wird der Header tatsaechlich abgespalten
	if ($x > -1)
	{
		$buf = substr($buf,$x + $split_length);
	}

	close $socket;

	$buf;
} # Ende der Subroutine RetrieveHTTP


############################################################
#
# subroutine: OpenSocket
#   Verwendung:
#     $socket_handle = &OpenSocket($host, $port);
#
#   Parameter:
#     $host = Hostname, mit dem zu verbinden ist
#     $port = Port, mit dem zu verbinden ist
#
#   Ausgabe:
#     Ein Handle zu dem geoeffnet Socket
#
############################################################

sub OpenSocket
{
	local($hostname, $port) = @_;

	local($ipaddress, $fullipaddress, $packconnectip);
	local($packthishostip);
	local($AF_INET, $SOCK_STREAM, $SOCK_ADDR);
	local($PROTOCOL, $HTTP_PORT); 

	# Die folgendende Variablen werden entsprechend den Werten
	# definiert, die in der Bibliothek socket.pm deklariert sind. Wenn
	# noch PERL 4 installiert ist und daher keine Socket-Bibliothek existiert,
	# koennen einige Standardwerte eingesetzt werden. Z.B.:
	# 2 fuer AF_INIT, 1 fuer SOCK_STREAM.
	# Wenn 1 fuer SOCK_STREAM nicht funktioniert , versuchen Sie 
	# es mit 2.
	#
	# AF_INET definiert die Internet-Klasse der Adressierung
	#
	# SOCK_STREAM ist eine Variable, die das Programm anweist, eine
	# Socket-Verbindung zu benutzen. Dies weicht von SOCK_DGRAM
	# ab, das unter Verwendung verbindungsloser Paradigmen statt
	# dessen UDP Datagramme senden wuerde.
	# 
	# PROTOCOL ist TCPIP (6).
	#
	# PORT sollte fuer den HTTP-Zugriff im allgemeinen 80 sein; einige
	# Sites benutzen allerdings alternative Ports  wie 8080.
	#
	# SOCK_ADDR ist die vollstaendige Socket-Adresse im Paketformat -
	# einschliesslich den Werten von AF_INIT, HTTP_PORT und IP ADDRESS
	# (in dieser Reihenfolge).
	#

	$AF_INET = AF_INET;
	$SOCK_STREAM = SOCK_STREAM;

	$SOCK_ADDR = "S n a4 x8";

	# Die folgenden Routinen ermitteln die Protokoll-Information

	$PROTOCOL = (getprotobyname('tcp'))[2];

	$HTTP_PORT = $port;

	# Standardport 80
	$HTTP_PORT = 80 unless ($HTTP_PORT =~ /^\d+$/);

	# Standardprotokoll TCP/IP
	$PROTOCOL = 6 unless ($PROTOCOL =~ /^\d+$/);

	# $ipaddress ist die Adresse des Hosts, zu dem eine Verbindung aufgebaut werden soll.
	$ipaddress = (gethostbyname($hostname))[4];

	$fullipaddress = join (".", unpack("C4", $ipaddress));

	$packconnectip = pack($SOCK_ADDR, $AF_INET, $HTTP_PORT, $ipaddress);
	$packthishostip = pack($SOCK_ADDR, $AF_INET, 0, "\0\0\0\0");

	# Zuerst wird der Socket zugewiesen.
	socket (S, $AF_INET, $SOCK_STREAM, $PROTOCOL) || &web_error( "Kann socket nicht erstellen: $!\n");

	# Dann erfolgt das Anbinden des Socket an den oertlichen Host
	bind (S,$packthishostip) || &web_error( "Kann Socket nicht an oertlichen Host anbinden anbinden: $!\n");

	# Dann wird der Socket mit dem entfernten Host verbunden.
	connect(S, $packconnectip) || &web_error( "Kann keine Socket-Verbindung zu $ipaddress erstellen: $!\n");

	# Das folgende waehlt den Socket-Handle und stellt die Pufferung der Ausgabe ab.

	select(S);
	$| = 1;
	select (STDOUT);

	S;
} # Ende der Subroutine OpenSocket


############################################################
#
# subroutine: read_sock
#   Verwendung:
#	&read_socket(SOCKET_HANDLE, $timeout);
#
#   Parameter:
#	SOCKET_HANDLE = Handle zu einem zugewiesenen Socket
#	$timeout = Zeitspanne, die read_sock vor dem Timeout auf 
#		einen Dateneingang warten darf (Sekunden). 
#
#   Ausgabe:
#	Der Puffer enthaelt das, was ueber den Socket gelesen wurde.
# 
############################################################

sub read_sock
{
	local($handle, $endtime) = @_;
	local($localbuf,$buf);
	local($rin,$rout,$nfound);

	# $endtime wird auf time + $endtime gesetzt.
	$endtime += time;

	# Puffer leeren
	$buf = "";

	# $rin lschen (Leseeingangsvariable)
	$rin = '';
	# $rin als Vektor des Socket-Filehandle definieren
	vec($rin, fileno($handle), 1) = 1;

	# $nfound ist 0, da noch nichts gelesen worden ist.
	$nfound = 0;

	# Schleife bis zum timeout oder bis etwas gelesen wurde.
	read_socket: 
	while (($endtime > time) && ($nfound <= 0))
	{
		# 1024 Bytes auf einmal lesen
		$length = 1024;

		# Puffer vorab zuweisen
		$localbuf = " " x 1025;

		# NT unterstuetzt kein Polling, um zu sehen, ob es irgendwelche
		# Zeichen zu empfangen gibt. Dies ist wichtig, weil der Rechner
		# nicht blockiert werden soll, wenn es nichts zu empfangen gibt.

		$nfound = 1;
		if ($http_os ne "NT")
		{
			# Das folgende fuert ein Polling durch, um zu sehen, ob es
			# im Eingangspuffer irgend etwas zu lesen gibt. Falls ja, 
			# wird spaeter die sysread-Routine aufgerufen.
			$nfound = select($rout=$rin, undef, undef,.2);
		}
	}
        
	# Wenn im Lesepuffer etwas gefunden wurde, sollte es unter Verwendung 
	# von sysread gelesen werden.
	if ($nfound > 0)
	{
		$length = sysread($handle, $localbuf, 1024);
		if ($length > 0)
		{
			$buf .= $localbuf;
		}
	}

	# Rueckgabe des Pufferinhalts.
	$buf;
}


############################################################
#
# subroutine: web_error
#   Verwendung:
#	&web_error("Datei xxx konnte nicht geoeffnet werden");
#
#   Parameter:
#	$error = Beschreibung des Webfehlers
#
#   Ausgabe:
#	Keine
# 
############################################################

sub web_error
{
	local ($error) = @_;
	$error = "Fehler: $error";
	# print "$error<p>\n";

	# Die beendet das Programm vorzeitig und gibt einen
	# Fehler an stderr aus.

	return "$error<p>\n";

} # end of web_error

1;

